from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker

from voxu.models import get_http_request_log_table_class, Base


class DatabaseConfigurator:

    def __init__(self, app):
        self.voxu_table = app.config.get('VOXU_LOG_TABLE', 'http_request_logs')
        self.database_url = app.config.get('SQLALCHEMY_DATABASE_URI', None)
        if not self.database_url:
            raise ValueError("SQLALCHEMY_DATABASE_URI is not set in the app's configuration")

    def init_db(self):
        engine = create_engine(self.database_url)

        # Ensure the table exists
        if not engine.dialect.has_table(engine.connect(), self.voxu_table):
            HTTPRequestLog = get_http_request_log_table_class(self.voxu_table)
            Base.metadata.create_all(engine, tables=[HTTPRequestLog.__table__])

        db_session = scoped_session(sessionmaker(autocommit=False,
                                                 autoflush=False,
                                                 bind=engine))
        return db_session

    def close_db(self, app, db_session):
        @app.teardown_appcontext
        def shutdown_session(response_or_exc):
            db_session.remove()
            return response_or_exc



from flask import Blueprint, render_template, current_app

from .database import DatabaseConfigurator
from .models import get_http_request_log_table_class

voxu_blueprint = Blueprint('voxu', __name__, url_prefix='/voxu', template_folder='templates')


@voxu_blueprint.route('/logs')
def logs():
    HTTPRequestLog = get_http_request_log_table_class()
    db_session = DatabaseConfigurator(current_app).init_db()
    logs = db_session.query(HTTPRequestLog).all()
    db_session.close()
    return render_template('logs.html', logs=logs)



from datetime import datetime

from sqlalchemy import Column, Integer, String, Text, DateTime
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()


def get_http_request_log_table_class(table_name='http_request_logs'):
    class HTTPRequestLog(Base):
        __tablename__ = table_name
        __table_args__ = {'extend_existing': True}
        id = Column(Integer, primary_key=True)
        method = Column(String(10))
        ip_address = Column(String(15))
        url = Column(Text)
        headers = Column(Text)
        body = Column(Text)
        response_status = Column(Integer)
        response_headers = Column(Text)
        response_body = Column(Text)
        timestamp = Column(DateTime, default=datetime.utcnow)

    return HTTPRequestLog



import json

from werkzeug.wrappers import Request, Response

from voxu.models import get_http_request_log_table_class


class RequestResponseLoggingMiddleware:
    def __init__(self, app, db_session):
        self.app = app
        self.HTTPRequestLog = get_http_request_log_table_class()
        self.db_session = db_session

    def __call__(self, environ, start_response):
        # Wrap the incoming environment in a request and handle it
        request = Request(environ)
        response_body = []
        response_headers = []
        status_code = []
        original_start_response = start_response

        # Define a function to capture response data
        def custom_start_response(status, headers, exc_info=None):
            status_code.append(status)
            response_headers.extend(headers)
            return original_start_response(status, headers, exc_info)

        response = self.app(environ, custom_start_response)

        # Construct the response object
        response_ = Response(
            response=b''.join(response_body),
            status=status_code[0],
            headers=response_headers
        )

        # Log the request and response
        self.log_request_response(request, response_)

        return response

    def log_request_response(self, request, response):
        # Create log entry
        log_entry = self.HTTPRequestLog(
            ip_address=request.remote_addr,
            method=request.method,
            url=request.url,
            headers=json.dumps(dict(request.headers)),
            body=request.get_data(as_text=True),
            response_status=response.status_code,
            response_headers=json.dumps(dict(response.headers)),
            response_body=response.get_data(as_text=True)
        )
        self.db_session.add(log_entry)
        try:
            self.db_session.commit()
        except Exception as e:
            self.db_session.rollback()
            raise e
        finally:
            self.db_session.close()



from setuptools import setup, find_packages

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='voxu',
    version='0.1.5',
    packages=find_packages(),
    install_requires=[
        'sqlalchemy',
        'flask',
        'psycopg2-binary'
    ],
    keywords=['Python', 'Flask', 'Logging', 'SQLAlchemy', 'PostgreSQL'],
    url='https://github.com/yorubadeveloper/voxu',
    author='Yoruba Developer',
    long_description=long_description,
    long_description_content_type='text/markdown'
)



from voxu.database import DatabaseConfigurator
from voxu.models import Base, get_http_request_log_table_class


class Voxu:
    def __init__(self, app=None):
        self.app = app
        self.database_configurator = None
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.database_configurator = DatabaseConfigurator(app=app)
        db_session = self.database_configurator.init_db()

        # Import and initialize middleware
        from .middleware import RequestResponseLoggingMiddleware
        app.wsgi_app = RequestResponseLoggingMiddleware(app.wsgi_app, db_session)

        # Import and initialize blueprint
        from .voxu_ui import voxu_blueprint
        app.register_blueprint(voxu_blueprint)

        import atexit
        atexit.register(self.database_configurator.close_db, app, db_session)
